/* 
 Copyright (C) 2020 Michał Gańko <flexphperia(at)gmail.com>
 @license: This file can not be copied, modified and/or distributed without the express permission of author.
 */

+function ($) {
    "use strict";

//controller base with methods do dispatch events
    window.emiterBase = {

        //callbacks storage
        _callbacks: null,

        /**
         * Listen on the given `event` with `fn`.
         *
         */
        on: function (event, fn) {
            this._callbacks = this._callbacks || {};
            (this._callbacks['$' + event] = this._callbacks['$' + event] || []).push(fn);
            return this;
        },

        /**
         * Adds an `event` listener that will be invoked a single
         * time then automatically removed.
         *
         */
        once: function (event, fn) {
            function on() {
                this.off(event, on);
                fn.apply(this, arguments);
            }

            on.fn = fn;
            this.on(event, on);
            return this;
        },

        /**
         * Remove the given callback for `event` or all
         * registered callbacks.
         *
         */
        off: function (event, fn) {
            this._callbacks = this._callbacks || {};

            // all
            if (0 == arguments.length) {
                this._callbacks = {};
                return this;
            }

            var regexpString = '',
                namespacePos = event.indexOf(':');

            if (namespacePos == 0) { //begins with namespace, delete all events listeners with this nemaspece, tested
                regexpString = event + '$'; //ends with 
            } else if (namespacePos == -1) { //no namepsace found, tested
                regexpString = '^\\$' + event; //begins with
            } else {//with namespace or not
                regexpString = '^\\$' + event + '$'; //exactly this
            }

            //iterate over all
            for (var prop in this._callbacks) {
                if (new RegExp(regexpString).test(prop)) {
                    if (!fn) {
                        delete this._callbacks[prop];
                    } else { //remove specific callback
                        for (var i = this._callbacks[prop].length - 1; i >= 0; i--) {
                            var cb = this._callbacks[prop][i];
                            if (cb === fn) {
                                this._callbacks[prop].splice(i, 1);
                                break;
                            }
                        }
                    }
                }
            }

            return this;
        },

        /**
         * Emit `event` with the given args.
         *
         */
        emit: function (event) {
            this._callbacks = this._callbacks || {};
            var args = [].slice.call(arguments, 1),
                    callbacks = [];

            for (var prop in this._callbacks) {
                var withoutNamespace = prop.split(':')[0];

                if (withoutNamespace == '$' + event)//match
                    callbacks = callbacks.concat(this._callbacks[prop]);
            }

            var prevented = false;
            
            if (callbacks) {
                callbacks = callbacks.slice(0); //make copy, one of callbacks may unregister itself
                for (var i = 0, len = callbacks.length; i < len; ++i) {
                    prevented = callbacks[i].apply(this, args) === false || prevented;
                }
            }

            return !prevented;
        },

        /**
         * Return array of callbacks for `event`.
         *
         */
        listeners: function (event) {
            this._callbacks = this._callbacks || {};
            return this._callbacks['$' + event] || [];
        }


    }

}($);