

+function($){
"use strict";

/* 
 * Util for creating form validators.
 * It adds method for checking that form is dirty or not.
 */
window.formValidator = {

    ignoredFieldSelector : '.val-ignored',
    //without buttons that are toggles for dropdown,m unhighlight field will break dropdown fuctionality
    inputSelector : ':input:not(button[data-toggle="dropdown"])',

    filterMethods : {}, //array of filter methods
    validationMethods : {}, //array of validation validationMethods
    validationMessages : {}, //validation validationMessages to validationMethods

    //set some default for validation plugin
    init : function (){
        
        mainCtrl.$body.tooltip({
            selector: '[data-toggle="tooltip"]', //specify the selector here
            html: true,
            placement: 'bottom'
        });

        mainCtrl.$body.on('show.bs.tooltip', '[data-toggle="tooltip"]', function(event) {
            var tooltip = $(event.target).data('bs.tooltip');
            $(tooltip.tip).addClass('tooltip-danger')
        });
        
        this.prepareValidationMethods();
        this.prepareValidationMessages();
        
        this.prepareFilterMethods();
    },
    
    
    prepareValidationMethods : function(){
        //order matters
        
        this.validationMethods.required = function(value, $field){
            return value.trim().length > 0;
        };
        
        this.validationMethods.minlen = function(value, $field, len){
            return value == '' || value.trim().length >= len;
        };
        
        this.validationMethods.maxlen = function(value, $field, len){
            return value == '' || value.trim().length <= len;
        };
        
        this.validationMethods.number = function( value, $field ) {
            return /^(?:-?\d+|-?\d{1,3}(?:,\d{3})+)?(?:\.\d+)?$/.test( value );
        };
        
        this.validationMethods.min = function(value, $field, param){
            return value >= param;
        };
        
        this.validationMethods.max = function(value, $field, param){
            return value <= param;
        };
        
        // https://jqueryvalidation.org/step-method/
	this.validationMethods.step = function( value, $field, param ) {
            var type = $field.attr( "type" ),
                errorMessage = "Step attribute on input type " + type + " is not supported.",
                supportedTypes = [ "text", "number", "range" ],
                re = new RegExp( "\\b" + type + "\\b" ),
                notSupported = type && !re.test( supportedTypes.join() ),
                decimalPlaces = function( num ) {
                        var match = ( "" + num ).match( /(?:\.(\d+))?$/ );
                        if ( !match ) {
                                return 0;
                        }

                        // Number of digits right of decimal point.
                        return match[ 1 ] ? match[ 1 ].length : 0;
                },
                toInt = function( num ) {
                        return Math.round( num * Math.pow( 10, decimals ) );
                },
                valid = true,
                decimals;

            // Works only for text, number and range input types
            // TODO find a way to support input types date, datetime, datetime-local, month, time and week
            if ( notSupported ) {
                throw new Error( errorMessage );
            }

            decimals = decimalPlaces( param );

            // Value can't have too many decimals
            if ( decimalPlaces( value ) > decimals || toInt( value ) % toInt( param ) !== 0 ) {
                    valid = false;
            }

            return valid;
        };
        
        //Hasło musi mieć minimalną długośc 6 znaków i zawierać przynajmniej jedną literę i cyfrę.
        this.validationMethods.pass = function(value, $field){    
            var patt = /^(?=.*[0-9]+.*)(?=.*[a-zA-ZáàâäãåçćéèêëęíìîïłñóòôöõśúùûüýÿæœżźÁÀÂÄÃÅÇĆÉÈÊËĘÍÌÎÏÑÓÒÔÖÕŚÚÙÛÜÝŸÆŒŻŹ]+.*)[0-9\p{L}\S]{6,}$/;
            return patt.test(value);
        };
        
        //letters, digists and dots, starts with letter, ends letter or digit
        this.validationMethods.username = function(value, $field){  
            var patt = /^[a-z]{1}[a-z|0-9|\.]+[a-z|0-9]{1}$/;
            return patt.test(value);
        };
        
        this.validationMethods.equalto = function(value, $field, param){
            return value === $field.closest('form').find(param).val();
        };
        
    },
    
    prepareValidationMessages : function(){
        
        this.validationMessages.required = YourCalendar.lang['valRequired'];
        this.validationMessages.minlen = YourCalendar.lang['valMinLength'];
        this.validationMessages.maxlen = YourCalendar.lang['valMaxLength'];
        this.validationMessages.min = YourCalendar.lang['valMin'];
        this.validationMessages.max = YourCalendar.lang['valMax'];
        this.validationMessages.step = YourCalendar.lang['valStep'];
        this.validationMessages.pass = YourCalendar.lang['valPassword'];
        this.validationMessages.equalto = YourCalendar.lang['valEqualTo'];
        this.validationMessages.username = YourCalendar.lang['valUsername'];

        this.validationMessages.formError = YourCalendar.lang['valFillFields'];
//        console.log(this.validationMessages.step);
    },
    
    //methods used when deserializing data into form or serializing from form
    prepareFilterMethods : function(){
        
    },
    
    validateForm : function($form){
        var self = this,
            isValid = true;
           
      
        //iterate over all inputs etc. 
        $form.find(this.inputSelector).each(function(){
            
            var fieldResult = self.validateField($(this));
            
            isValid = !isValid ? isValid : fieldResult; //if last element failed than store this in flag
        });
        
  
        if (isValid){
            this.unhighlightForm($form);
        }
        else{
            this.highlightForm($form);
        }
  
        
        return isValid;
    },
    
    
    validateField : function($field){
        
        if ($field.is(this.ignoredFieldSelector)){ //skip
            if ( $field.data('validation-error') ){ //might be highlighted earlier and now we do not need highlighting
                this.unhighlightField($field);
                $field.data('validation-error', false);
            }
            return true;
        }
        
        var self = this,
            isValid = true,
			failedMessage = '';
        
        //check for all validationMethods is field uses any
        $.each(this.validationMethods, function( methodName, func ) {
            
            //search that field has data attribute with validation method
            var dataValue = $field.data('val-' + methodName);
            var result = true;
            var shouldCheckMethod = dataValue === "" || dataValue; //data attribute exists without value or with value
            
            var fieldVal = $field.val();
            
            //if it exists, call method function with parameter and check is optional
            if ( shouldCheckMethod && fieldVal || (methodName == 'required' && shouldCheckMethod) ) {
                result = func(fieldVal, $field, dataValue); //call method
            }
            
            if (!result){ //failed validation
				failedMessage = self.formatMessage(methodName, dataValue);
                // self.highlightField($field, self.formatMessage(methodName, dataValue));
                // $field.data('validation-error', true);
                isValid = false;
                return false; //break loop
            }
            // else{
                // self.unhighlightField($field);
				// console.log($field.get(0), methodName);
                // $field.data('validation-error', false);
            // }
        });
		
		if (!isValid){
			self.highlightField($field, failedMessage);
		}
		else{
			self.unhighlightField($field);
		}
	
		$field.data('validation-error', isValid);

        
        return isValid;
    },
    
    //type can be to or from
    filterFieldValue : function($field, type, value){
       
        var val = type === 'to' ? value : $field.val(); //get input value
        
//        if (type === 'from' && !$field.is(':input'))
//            throw "Not supported on no input fields.";
        
        //filter values
        $.each(this.filterMethods, function( methodName, func ) {
            //search that field has data attribute with validation method
            var dataValue = $field.data('filter-' + methodName);
            var shouldUseMethod = dataValue === "" || dataValue;

            //if it exists,
            if ( shouldUseMethod  ) {
                val = func[type](val, $field); //call method
            }
        });
        
        return val;
    },
    
    //show error
    highlightField : function($field, message){
        
        //has delegation to show error tooltip on other element
        if ( $field.is('[data-val-delegate-sibling]') ){
            $field = $field.siblings($field.data('val-delegate-sibling'));
        }
//        alert(message);
        
        $field.addClass('is-invalid');
        
        //set tooltip in data-original-title
        $field.attr('data-toggle', 'tooltip').attr('data-original-title', message);
    },
    
    //remove error
    unhighlightField : function($field){
             // console.log($field.get(0));
         if ( $field.is('[data-val-delegate-sibling]') ){
            $field = $field.siblings($field.data('val-delegate-sibling'));
        }
        
        
        $field.removeClass('is-invalid');
        
        $field.removeAttr('data-toggle data-original-title');
    },
    
    
    highlightForm : function($form){
        var $alert = $form.find('.alert').eq(0);
  
        $alert.text(this.validationMessages.formError);
        $alert.removeClass('d-none');
    },
    
    unhighlightForm : function($form){
        var $alert = $form.find('.alert').eq(0);
        
        $alert.addClass('d-none');
    },

    
    
    formatMessage : function(methodName, param){
        return this.validationMessages[methodName].replace('{0}', param);
    },
    
    //todo cleanup, move functions to out
    serializeForm : function($form){
        var o = {},
            pushes = {}, //stores how many pushes we have
            self = this;
      
        var patterns = {
            allowedInputs : /checkbox|radio|text|range|number|hidden|password|search|select|textarea/i,
            key:      /[a-z0-9_]+|(?=\[\])/gi,
            push:     /^$/,
            fixed:    /^\d+$/,
            named:    /^[a-z0-9_]+$/i
        };
                
        var inputsArray = $form.find(':input');
//        var inputsArray = $form.map(function() {
//                        return $.makeArray(this.elements); //find all elements
//                    });
                        
        inputsArray = inputsArray.filter(function() {
            //must have name and be in allowed types (when not filtered it contain buttons too)       
            return this.name && patterns.allowedInputs.test(this.type);       
        }); 
        
        
        //map array to find values and store in objects
        inputsArray = inputsArray.map(function(i, elem) {
            
            //only add radio button that is checked
            if (/radio/i.test(elem.type) && !elem.checked){
                return; //skip
            }
            var $input = $(elem);

            var val = self.filterFieldValue($input, 'from');
            
             
            if (elem.type === "checkbox"){
                val = elem.checked ? 1 : 0; //set proper value
            }
            
            if ($.isArray(val)){ //can be multiple values from select
                val = val.map(function(idx, value) {
                    return {
                        element: elem,
                        value: value
                    };
                }); 
            }
            else{
                return {
                    element: elem,
                    value: val
                };
            }
        });
        
        
        var build = function(base, key, value) {
//            console.log(base, key, value);
            base[key] = value;
            return base;
        };
          
        var incrementPush = function (key) {
            if (pushes[key] === undefined) {
              pushes[key] = 0;
            }
            return pushes[key]++;
        };
        
        var makeObject = function(inputObj) {

            var value = inputObj.value,
                keys = inputObj.element.name.match(patterns.key),
                k;

            // nest, nest, ..., nest
            while ((k = keys.pop()) !== undefined) {
              // foo[]
              if (patterns.push.test(k)) {
                var idx = incrementPush(inputObj.element.name.replace(/\[\]$/, ''));
                value = build([], idx, value);
              }

              // foo[n]
              else if (patterns.fixed.test(k)) {
                value = build([], k, value);
              }

              // foo; foo[bar]
              else if (patterns.named.test(k)) {
                value = build({}, k, value);
              }
            }

            return value;
        };
        
//        var self = this;
        
//        var encode = function (inputObj) {
//            //check for all filterMethods is field uses any
//            $.each(self.filterMethods, function( methodName, func ) {
//                var $input = $(inputObj.element);
//                //search that field has data attribute with validation method
//                var dataValue = $input.data('filter-' + methodName);
//                var shouldUseMethod = dataValue === "" || dataValue;
//
//                //if it exists,
//                if ( shouldUseMethod  ) {
//                    inputObj.value = func['from'](inputObj.value, $input); //call method
//                }
//            });
//            
//            return inputObj.element.type === "checkbox" ? (inputObj.element.checked ? 1 : 0) : inputObj.value;
//        };   

        for (var i = 0, len=inputsArray.length; i<len; i++) { 
            var inputObj = inputsArray[i];
            
            var obj = makeObject(inputObj);
            o = $.extend(true, o, obj);
        }
        
        
//        console.log(o);
//        return;
        return o;
    },
    
    deserializeForm : function($form, data){
        if (!data)
            return;
        
        var self = this;
 
        $('[name]', $form).each(function( index ){
            var $input = $(this),
                name = $input.attr('name'),
                type,
                value;
                
//                console.log(name, data);
//                console.log(name);
            //not found in object
            if ( !data || !data.hasOwnProperty( name )  )
                return;

            value = data[name];
            
            value = value === null ? '' : value;
            
            //check for all filterMethods is field uses any
            value = self.filterFieldValue($input, 'to', value);
            
//            $.each(self.filterMethods, function( methodName, func ) {
//                //search that field has data attribute with validation method
//                var dataValue = $input.data('filter-' + methodName);
//                var shouldUseMethod = dataValue === "" || dataValue;
//
//                //if it exists,
//                if ( shouldUseMethod  ) {
//                    value = func['to'](value, $input); //call method
//                }
//            });

            type = $input.attr('type');

            if (type == 'checkbox' ){
                $input.prop('checked', Number(value) ? true : false);
            }
            else if (type == 'radio'){
                if ($input.attr('value') == value)
                    $input.prop("checked", true);
            }               
            else{ //all text inputs, selects    
//                value = String(value).replace(/_NL_/g, "\n"); //replace back to new lines, used in image comment data attribute reading
//console.log($input.get(0));
                $input.val(value );
            }
        });
    },
    
    
    resetForm : function($form){
        var self = this;
        
        $form.find(this.inputSelector).each(function(){
            self.unhighlightField($(this));
        });
        
        $form.resetForm();
        
        $form.find('.alert').eq(0).addClass('d-none');

        this.cleanFormDirty($form);
    },
    
    
    cleanFormDirty : function ($form){
        $form.data('serializedData', this.serializeForm($form));
//        console.l
//        this.savedData = this.serializeForm($form);
    },
    
    
    isFormDirty : function ($form){
        var savedData = $form.data('serializedData');

        if (JSON.stringify(savedData) != JSON.stringify(this.serializeForm($form)))
            return true;
        else
            return false;
    }

};



}($);

