/**
 * This file is part of Your Calendar app. It requires license to use.
 * Copyright (c) 2022 flexphperia.net All rights reserved.
 * 
 * @preserve
 */

+function ($) {
    "use strict";

    /* Plugin for reseting forms to its initiall state */
    $.fn.resetForm = function () {
        return this.each(function () {
            var $form = $(this);


            //reset all  inputs
            $(':input', $form).not(':button, :submit, :reset').each(function () {
                var $input = $(this);

//            console.log($input.get(0));
                //when select, select first option
                if ($input.is('select')) {
                    $input.val($input.find('option:first').val());
                    return;
                }

                if ($input.is('[type="checkbox"]')) {
                    $input
                            .prop('checked', false) // Unchecks it
                            .removeAttr('checked');
                    return;
                }

                if ($input.is('[type="radio"]')) {
                    var name = $input.attr('name');
                    $input
                            .prop('checked', false) // Unchecks it
                            .removeAttr('checked');

                    $input.closest('form').find('[type="radio"][name="' + name + '"]').first().prop('checked', true); //check fiorst
                    return;
                }

//            console.log($input.attr('name'));
                $input.val('');
            });
        });
    };


    var mainCtrl = {

        _initProperties: function () {
            this.$window = $(window);
            this.$document = $(document);
            this.$html = $('html');
            this.$body = $('body');
            this.$sizeMeter = $('<div style="position: fixed; top: 0; bottom: 0; left: 0; right: 0"></div>');
            this.$header = $('#header');

            //uset to get window size, better than inner height, cross browser
            this.$sizeMeter
                    .hide()
                    .appendTo(this.$body);

            this.$html.addClass(window.devicePixelRatio > 1 ? 'retina' : 'no-retina');

            this._testFeatures();

            this.win = {
                minWidth: parseInt(this.$body.css('min-width')),
                minHeight: parseInt(this.$body.css('min-height')),
                safeWidth: 0, //safe if less than min is min
                safeHeight: 0,
                scrollBarWidth: this._getBrowserScrollbarWidth(),
                resizing: false,
                resizingWidth: false,
                resizingHeight: false,
                _oldSafeHeight: 0,
                _oldSafeWidth: 0
            };
        },

        init: function () {
            this._initProperties();
//            this.updateMenuSelection();

            this.checkVersion();

            //prevent on ios 10 page scaling - needed for proper scaling images in gallery
            if (/iPad|iPhone|iPod/.test(navigator.userAgent)) {
                this.$document.find('head').append('<style>*{cursor:pointer;-webkit-tap-highlight-color:rgba(0,0,0,0)}</style>');
                this.$window.on('gesturestart touchmove', function (evt) {
                    if (evt.originalEvent.scale !== 1) {
                        evt.originalEvent.preventDefault();
                        this.$body[0].style.transform = 'scale(1)';
                    }
                });
            }

            this.$window.resize(this.debounce($.proxy(this.resizeHandler, this), 100));

            //show pending message if any
            var pendingMessage = mainCtrl.readState('pendingMessage');

            if (pendingMessage) {
                mainCtrl.showNotify(pendingMessage);
                mainCtrl.removeState('pendingMessage');
            }

        },

        resizeHandler: function (event, onlyWinUpdate) {

            //window innerHeight is falling on some browsers (android chrome),
            //show fixed div and measure its size, its better than window.innerHeight but smoetimes still buggy
            this.$sizeMeter.show();

            this.win.safeWidth = Math.max(Math.round(this.$sizeMeter.width()), this.win.minWidth);
            this.win.safeHeight = Math.max(Math.round(this.$sizeMeter.height()), this.win.minHeight);

            this.$sizeMeter.hide();


            //check what is changed
            if (this.win.safeWidth != this.win._oldSafeWidth) {
                this.win._oldSafeWidth = this.win.safeWidth;
                this.win.resizingWidth = true;
            }

            if (this.win.safeHeight != this.win._oldSafeHeight) {
                this.win._oldSafeHeight = this.win.safeHeight;
                this.win.resizingHeight = true;
            }

            this.win.resizing = true;

//        console.log('resize event', this.win.resizing, this.win.resizingWidth, this.win.resizingHeight, this.win._oldSafeWidth, this.win._oldSafeHeight);
            if (!onlyWinUpdate)
                this.emit('resize');

            //reset after emiting event
            this.win.resizing = false;
            this.win.resizingWidth = false;
            this.win.resizingHeight = false;
        },

        /* notify funciton, shows messages about saving, loading operations*/
        showNotify: function (message, type, dontHide) {
            type = type ? type : 'success';

            var times = {
                success: 2000,
                info: 5000,
                danger: 5000
            };

            $.toast({
                title: message,
                type: type,
                delay: dontHide ? -1 : times[type],
            });
        },

        // Returns a function, that, as long as it continues to be invoked, will not
        // be triggered. The function will be called after it stops being called for
        // N milliseconds. If `immediate` is passed, trigger the function on the
        // leading edge, instead of the trailing.
        debounce: function (func, wait, immediate) {
            var timeout, args, context, timestamp, result;
            return function () {
                context = this;
                args = arguments;
                timestamp = new Date();
                var later = function () {
                    var last = (new Date()) - timestamp;
                    if (last < wait) {
                        timeout = setTimeout(later, wait - last);
                    } else {
                        timeout = null;
                        if (!immediate)
                            result = func.apply(context, args);
                    }
                };
                var callNow = immediate && !timeout;
                if (!timeout) {
                    timeout = setTimeout(later, wait);
                }
                if (callNow)
                    result = func.apply(context, args);
                return result;
            };
        },

        // Returns a function, that, when invoked, will only be triggered at most once
        // during a given window of time. Normally, the throttled function will run
        // as much as it can, without ever going more than once per `wait` duration;
        // but if you'd like to disable the execution on the leading edge, pass
        // `{leading: false}`. To disable execution on the trailing edge, ditto.
        throttle: function (func, wait, options) {
            var context, args, result;
            var timeout = null;
            var previous = 0;
            if (!options)
                options = {};
            var later = function () {
                previous = options.leading === false ? 0 : new Date().getTime();
                timeout = null;
                result = func.apply(context, args);
                if (!timeout)
                    context = args = null;
            };
            return function () {
                var now = new Date().getTime();
                if (!previous && options.leading === false)
                    previous = now;
                var remaining = wait - (now - previous);
                context = this;
                args = arguments;
                if (remaining <= 0 || remaining > wait) {
                    clearTimeout(timeout);
                    timeout = null;
                    previous = now;
                    result = func.apply(context, args);
                    if (!timeout)
                        context = args = null;
                } else if (!timeout && options.trailing !== false) {
                    timeout = setTimeout(later, remaining);
                }
                return result;
            };
        },

        writeState: function (key, value) {
            var storage = false;
            try {
                storage = window.localStorage;


                storage.setItem(key, value);
            } catch (_error) {

            }
        },

        removeState: function (key, value) {
            var storage = false;
            try {
                storage = window.localStorage;


                storage.removeItem(key);
            } catch (_error) {

            }
        },

        readState: function (key) {
            var storage = false,
                    result = false;

            try {
                storage = window.localStorage;

                result = storage.getItem(key);
            } catch (_error) {

            }

            return result;
        },

//        redirect: function (to, message) {
//            this.pendingMessage(message);
//
//            window.location.href = YourCalendar.siteUrl + to; //redirect
//        },
        
        
        historyBack: function (message) {
            this.pendingMessage(message);

            history.back();
        },
        
        pendingMessage: function (message) {
            if (message)
                this.writeState('pendingMessage', message);
        },

        /*
         * small util function used for stateless divs
         * div may contain elements that have data-state="state name" attribute
         * if you wanna show only elements with definied state call this function
         */
        applyViewState: function ($context, stateName) {
//        console.log($('*[data-state~="' + stateName + '"]', $context).length)
            $('*[data-state]', $context).addClass('d-none');
            //clear or states
            $('*[data-state~="' + stateName + '"]', $context).removeClass('d-none');
            //hide not used
        },

        flexWrappedHelper: function ($flexEl) {
            var wrapped = false;

            $flexEl
                    .removeClass('wrapped')
                    .find('> *:visible')
                    .removeClass('first-in-row last-in-row') //must remove here
                    .each(function () {
                        var $currentItem = $(this);
                        var $prevItem = $currentItem.prev();
                        var $nextItem = $currentItem.next();

                        //cast to proper value
                        // $prevItem = $prevItem.length ? $prevItem : false;
                        // $nextItem = $nextItem.length ? $nextItem : false;

                        // console.log($nextItem);

                        // $currentItem.removeClass('first-in-row last-in-row');
                        var current_wrapped = $prevItem.length && ($currentItem.offset().top >= $prevItem.offset().top + $prevItem.outerHeight());

                        // if ($prevItem.length)
                        // console.log($currentItem.offset().top,  $prevItem.offset().top + $prevItem.outerHeight());


                        if (current_wrapped || !$prevItem.length) {
                            $currentItem.addClass('first-in-row');

                        }

                        //not first element at all and is wrapped so prev is last row element
                        if ($prevItem.length && current_wrapped) {
                            $prevItem.addClass('last-in-row');
                            wrapped = true;
                        }

                        //no next element
                        if (!$nextItem.length) {
                            $currentItem.addClass('last-in-row');
                        }
                    });

            if (wrapped)
                $flexEl.addClass('wrapped');
        },

        scrollToEl: function ($element, offset, callback) {
            var offset = !offset ? 0 : offset,
                scollPos = $element.offset().top - 59 - offset; //90 must be smaller than in onscroll 100

            if (this._getBodyScrollTop() == scollPos)
                return;

            this.$body.add(this.$html).stop().animate({scrollTop: scollPos}, 300).promise().then(function () {
                callback && callback();
            });
        },

        inViewport: function ($el) {
            var elementTop = $el.offset().top;
            var elementHeight = $el.outerHeight();
            var elementBottom = elementTop + elementHeight;

            var viewportTop = this._getBodyScrollTop();
            var viewportHeight = this.$window.height();
            var viewportBottom = viewportTop + viewportHeight;

//        console.log(elementTop >= viewportTop && elementBottom <= viewportBottom);
//        return elementBottom > viewportTop && elementTop < viewportBottom;
            return (elementTop >= viewportTop && elementTop < viewportBottom) &&
                    (elementBottom > viewportTop && elementBottom <= viewportBottom);

//        return !(viewportBottom < elementTop || viewportTop >elementBottom);
        },
        
        checkVersion : function(){

            if (!YourCalendar.checkVersion)
                return;

            $.ajax({
                url: 'https://flexphperia.net/yc/api/last_version/',
                dataType: 'jsonp',
                success: function(response) {
                    if (Number(response['last_version']) > Number(YourCalendar.version))
                        mainCtrl.showNotify(YourCalendar.lang.newVersionMessage, 'info');
                }
            });
        },
        

        _testFeatures: function () {
            var doc = window.document,
                el = doc.createElement("i"),
                style = el.style,
                prefs = "-webkit-".split(" "),
                domPrefs = "Webkit".split(" "),
                testAll = function (prop) {
                    var camel = prop.charAt(0).toUpperCase() + prop.substr(1),
                            props = (prop + " " + domPrefs.join(camel + " ") + camel).split(" ");

                    return !!testProps(props);
                },
                testProps = function (props) {
                    for (var i in props) {
                        if (style[props[i]] !== undefined) {
                            return true;
                        }
                    }

                    return false;
                };

            this.retina = this.$html.hasClass('retina');
            this.csstransforms = testAll("transform");
            this.csstransitions = testAll("transition");
            this.svg = !!document.createElementNS && !!document.createElementNS("http://www.w3.org/2000/svg", "svg").createSVGRect;
            this.touch = ('ontouchstart' in window) || window.DocumentTouch && document instanceof DocumentTouch;
        },

        /**
         Returns browser's native scrollbar width
         @method getBrowserScrollbarWidth
         @return {Number} the scrollbar width in pixels
         @static
         @private
         */
        _getBrowserScrollbarWidth: function () {
            var outer, outerStyle, scrollbarWidth;
            outer = document.createElement('div');
            outerStyle = outer.style;
            outerStyle.position = 'absolute';
            outerStyle.width = '100px';
            outerStyle.height = '100px';
            outerStyle.overflow = 'scroll';
            outerStyle.top = '-9999px';
            document.body.appendChild(outer);
            scrollbarWidth = outer.offsetWidth - outer.clientWidth;
            document.body.removeChild(outer);
            return scrollbarWidth;
        },

        //cross browser, ios, android, desktop
        _getBodyScrollTop: function () {
            return Math.round(window.pageYOffset || document.body.scrollTop); //pageYoffset can return float
        }
        

    };

    window.mainCtrl = $.extend(true, {}, emiterBase, mainCtrl); //must be deep copy;

}($);