+function ($) {
    "use strict";

    var modalEventsEditCtrl = {

        $element: null,
        $eventsListEl: null,
        $formTemplate: null,
        $rowTemplate: null,
        $form: null,

        //currently edited row element
        $editedRow: null,

        sortable: null,
        mode: null,

        callbackYes: null,
        callbackCancel: null,
        itemType: null,
        editedDays: null, //array of objects {year,month,day}
        events: null, //array of objects with events from db
        
        isDirty: false, //is dirty flag

        init: function () {
            this.$element = $('#modalEventsEdit');
            this.$eventsListEl = this.$element.find('.events-list');
            this.$formTemplate = this.$element.find('[data-form-template]').remove();
            this.$rowTemplate = this.$element.find('[data-row-template]');
            

            var self = this;

            //on button or on event details itself
            this.$element.on('click', 'button[data-edit], .event', function () {
                var $row = $(this).closest('.event-row');
                
                if (self.$editedRow && self.$editedRow.is($row))
                    return;
                
                self.rowEditEnd(true);
                
                self.rowEditStart($row);
            });
            
            
               this.$element.on('click', 'button[data-delete]', function () {
                var $row = $(this).closest('.event-row');
                self.rowEditEnd(true);
                
                self.rowDelete($row);
            });


            this.$element.on('click', '.modal-footer button.btn-primary', function () {
                if (!self.validateEditedRow())
                    return;
                
                self.rowEditEnd(true); //submit form
                
                self.onSave();
                self.$element.modal('hide');
            });

            this.$element.on('click', 'button[data-add]', function () {
                self.rowAdd();
            });

            this.$element.on('hidden.bs.modal', function (e) {
                self.onClose();
            })

            //decheck all but one
            this.$element.on('change', 'input[name="is_main"]', function (e) {
                var $input = $(this);

                if (self.mode == 'edit'){
                    $input.prop('checked', true);

                    self.$eventsListEl.find('input[name="is_main"]').not($input).prop('checked', false);
                }
                if (self.mode == 'add' && $input.prop('checked')){
                    self.$eventsListEl.find('input[name="is_main"]').not($input).prop('checked', false);
                }
                
                self.isDirty = true;
            });

        },
        

        open: function (mode, callbackYes, callbackCancel, editOptions) {
            var self = this;

            selectionCtrl.disable();

            this.mode = mode;
            this.callbackYes = callbackYes;
            this.callbackCancel = callbackCancel;

            mainCtrl.applyViewState(this.$element, this.mode);

            this.$element.modal({
                backdrop: 'static'
            });
            
            this.editedDays = editOptions.days;

            if (this.mode == 'edit') {

                //send only what is neeeded
                var params = $.extend({}, this.editedDays[0]); //get first
 
                params.item_id = editOptions.itemId;

                serviceCtrl.getEventsForDay( //save settings to server
                    params,
                    function(result){
                        self.events = result.events;
                        self.itemType = result.item_type;
                        self.$rowTemplate.find('.event-wrap').html(result.event_template);
                        //fill rows
                        $.each(self.events, function( index, event ) {
                            var $row = self.$rowTemplate.clone().removeClass('d-none');

                            self._prepareRowEl(event, $row);
                            self.$eventsListEl.append($row); 
                        });
                        self._prepareForm();
                    }
                );

                this.$element.find('.modal-title [data-date]').text(editOptions.dateString);
                
            } else if (this.mode == 'add') {
                
                serviceCtrl.getItemType( 
                    editOptions.itemId,
                    function(result){
                        self.itemType = result.item_type;
                        self.$rowTemplate.find('.event-wrap').html(result.event_template);
                        
                        self._prepareForm();
                    }
                ); 
                
                this.$element.find('.modal-title [data-num]').text(this.editedDays.length);
            }

            this.sortable = new Sortable(this.$eventsListEl.get(0), {
                animation: 100,
                handle: 'button[data-move]',
                direction: 'vertical',
                onUpdate: function () {
                    self.isDirty = true;
                }
            });
        },
        
        


        onSave: function () {
            this.rowEditEnd();

            var events = [],
                self = this;

            this.$eventsListEl.find('.event-row').each(function () {
                var $event = $(this);

                events.push(self.rowToEventObj($event));
            });
            
            
            if (this.isDirty)
                this.callbackYes && this.callbackYes(events); //only when dirty
            else
                mainCtrl.showNotify(YourCalendar.lang.no_changes);
        },

        onClose: function () {
            this.rowEditEnd();
            this.sortable.destroy();
            this.sortable = null;

            this.$eventsListEl.empty();
            this.$form.remove();
            this.$form = null;

            this.callbackYes = null;
            
            this.callbackCancel && this.callbackCancel();
            this.callbackCancel = null;
            this.item = null;
            this.events = null;
            this.editedDays = null;
            this.isDirty = false;

            selectionCtrl.enable();
        },

        rowEditStart: function ($rowEl) {
            

            
            //in editing mode
            this.rowEditEnd();

            this.$editedRow = $rowEl;

            var self = this;

            var $eventWrap = this.$editedRow.find('.event-wrap'),
                $collapse = $('<div class="collapse"></div>').append(this.$form),
                $eventSelect = this.$form.find('select[name="event_type"]');

//            this.$editedRow.find('.controls button[data-edit]').prop('disabled', true);

            $eventSelect.selectpicker({
                style: 'custom-select selectpicker',
                liveSearch: !mainCtrl.touch || $eventSelect.find('option').length >= 10 ? true : false,
                iconBase: '',
                sanitize: false,
                noneResultsText: $eventSelect.data('no-results')
            });

            //sets proper values of fields in form
            this._fillForm();
            
            var eventTypeId = $rowEl.find('input[name="type-id"]').val();
          
            var allowed = !eventTypeId || $eventSelect.find('option[value="'+ eventTypeId +'"]').length ? true : false;
//            
            this.$form.data('allowed-event-type', allowed);
            this.$form.find('.alert').toggleClass('d-none', allowed);

            if (allowed){
                
                //adding new row and only one event type is available, select is as default
                if (!eventTypeId && self.itemType.event_types.length == 1){ 
                    eventTypeId = self.itemType.event_types[0].id;
                }
                
                $eventSelect.selectpicker('val', eventTypeId); //set value
                $eventSelect.selectpicker('show');
            }
            else{
                $eventSelect.selectpicker('hide');
            }


            this.$form.on('submit', function (event) {
                event.preventDefault();
//                
                if (!self.validateEditedRow())
                    return;
//                
                self.isDirty = true;
                
                var formData = self._serializeForm();
                
                
                var icon = null,
                    eventName = null,
                    eventTypeId = formData.event_type;
                
                //only if allowed event editing update other things
                if (allowed){  
                    var eventTypeObj;
                    //search for event type selected
                    $.each(self.itemType.event_types, function( index, eventType ) {

                        if (eventType.id == eventTypeId){
                            eventTypeObj = eventType;
                            return false; //break loop
                        }
                    });

                    icon = eventTypeObj.icon;
                    eventName = eventTypeObj.name;
                }
                
                self._updateRowEl (
                    self.$editedRow, 
                    icon, //eventIcon, 
                    eventName, //eventName, 
                    eventTypeId, //eventTypeId, 
                    formData.description1, //desc1, 
                    formData.description2 //desc2, 
                    );

                self.rowEditEnd();
            });

            $eventWrap.find('.event').addClass('d-none');
            $eventWrap.append($collapse);

            $collapse.collapse();
        },

        rowEditEnd: function (acceptForm) {
            //not in editing mode
            if (!this.$editedRow)
                return;
            
            if (acceptForm){
                this.$editedRow.find('form').submit(); //try to submit
                return;
            }
            this.$form.off('submit');
            
            if (!this.validateEditedRow()){
                this.rowDelete(this.$editedRow);
            }
            
//            this.$editedRow.find('.controls button[data-edit]').prop('disabled', false);

            this.$editedRow.find('select[name="event_type"]')
                    .removeClass('is-invalid')
                    .selectpicker('destroy')
                    .find('option.bs-title-option').remove(); //fiexs selectpicker error that will not delete all things that should
            
            
            this.$editedRow.find('.collapse').remove();
            this.$editedRow.find('.event').removeClass('d-none');

            this.$editedRow = null;
            
        },

        rowDelete: function ($rowEl) {

            //if is main select other
            if (this.rowToEventObj($rowEl).is_main)
            {
//                console.log($rowEl.prev().get(0));
                $rowEl.siblings().eq(0).find('input[name="is_main"]').trigger('change');
            }
            $rowEl.remove();
            
            this.isDirty = true;
        },

        rowAdd: function () {
            if (!this.validateEditedRow())
                return;
            
            this.rowEditEnd(true); //submit form
            
            var $newRow = this.$rowTemplate.clone().removeClass('d-none');

            var obj = {
                is_main : false
            };

            //if first added in edit mode than its main
            if (this.mode == 'edit' && this.$eventsListEl.find('.event-row').length == 0){
                obj.is_main = true;
            }

            this._prepareRowEl(obj, $newRow); //to se proper id etc.

            this.$eventsListEl.append($newRow);
            this.rowEditStart($newRow);
            
            this.isDirty = true;
        },
        
        validateEditedRow: function () {
            if (!this.$editedRow)
                return true;
            
            var $form = this.$editedRow.find('form');
            var $select = $form.find('select');
            
            if (!$form.data('allowed-event-type')){
                return true;
            }
            
            if (!$select.selectpicker('val')) {
                $select.addClass('is-invalid');
                $select.selectpicker('setStyle'); //copies classof original element 
                return false;
            }
            
            return true;
        },

        rowToEventObj: function ($rowEl) {
            //used to convert html entities
//            var $tempConverter = $('<textarea />');


            var obj = {
                id : $rowEl.find('input[name="id"]').val(),
                is_main : $rowEl.find('input[name="is_main"]').prop('checked') ? 1 : 0,
                description1 : $rowEl.find('.description span[data-desc1] .value').text(),
                //description 2 might be still there 
                description2 : this.itemType.desc2_disabled ? '' : $rowEl.find('.description span[data-desc2] .value').text(),
                type : {id: $rowEl.find('input[name="type-id"]').val()}
            };
            
            return obj;
        },

        _prepareRowEl: function (obj, $rowEl) {
            var rand = 'a' + Math.floor(Math.random() * 101);
            $rowEl.find('input[name="is_main"]').attr('id', rand).siblings('label').attr('for', rand);

            $rowEl.find('input[name="is_main"]').prop('checked', obj.is_main);
            $rowEl.find('input[name="id"]').val(obj.id);
            
            this._updateRowEl (
                    $rowEl, 
                    obj.type ? obj.type.icon : null, //eventIcon, 
                    obj.type ? obj.type.name : null, //eventName, 
                    obj.type ? obj.type.id : null, //eventTypeId, 
                    obj.description1, //desc1, 
                    obj.description2 //desc2, 
                    );
            
        },
        
        _updateRowEl : function($rowEl, eventIcon, eventName, eventTypeId, desc1, desc2 ){
            
            if (eventTypeId){
                $rowEl.find('.event svg.icon').html('<use xlink:href="#' + eventIcon + '"></use>');
                $rowEl.find('.name').text(eventName);
                $rowEl.find('input[name="type-id"]').val(eventTypeId);
            }

            var $descriptionDiv = $rowEl.find('.description');
            
            desc1 = desc1 ? desc1 : '';           
            $descriptionDiv.find('span[data-desc1] .value')
                    .text(desc1);
//                    .prepend(desc1Label);
            
            desc2 = desc2 ? desc2 : '';
            $descriptionDiv.find('span[data-desc2] .value')
                    .text(desc2);

            $descriptionDiv.toggleClass('has-any-desc', this._has_any_desc(desc1, desc2));
            $descriptionDiv.toggleClass('desc1-empty', !desc1.length);
            $descriptionDiv.toggleClass('desc2-empty', !desc2.length);
        },
        
        _has_any_desc : function(desc1, desc2){
            var d1 = desc1 ;
            var d2 = this.itemType.desc2_disabled ? desc2 : '';

            if ( ('' + d1 + d2).length )
            {
                return true;
            }

            return false;
        },
        
        _prepareForm : function(){
            var $form = this.$formTemplate.clone().removeClass('d-none');
            //store
            var $descTypeGroups = $form.find('.form-group[data-desc-type]').remove();
            
            var $desc1Group = $descTypeGroups.filter('[data-desc-type="'+ this.itemType.desc1_type +'"]').clone();
            var desc1Id = 'a' + Math.floor(Math.random() * 999);
            var $desc1Label = $desc1Group.find('label');
            var $desc1Input = $desc1Group.find(':input');
            var desc1Label = this.itemType.desc1_label ? this.itemType.desc1_label : $desc1Label.data('default').replace('{0}', 1);
            $desc1Label.text(desc1Label);
            $desc1Label.attr('for', desc1Id);
            $desc1Input.attr('id', desc1Id);
            $desc1Input.attr('name', 'description1');
            $desc1Group.insertBefore($form.find('button'));
            
            if (!this.itemType.desc2_disabled){
                var $desc2Group = $descTypeGroups.filter('[data-desc-type="'+ this.itemType.desc2_type +'"]').clone();
                var desc2Id = 'a' + Math.floor(Math.random() * 999);
                var $desc2Label = $desc2Group.find('label');
                var $desc2Input = $desc2Group.find(':input');
                var desc2Label = this.itemType.desc2_label ? this.itemType.desc2_label : $desc1Label.data('default').replace('{0}', 2);
                $desc2Label.text(desc2Label);
                $desc2Label.attr('for', desc2Id);
                $desc2Input.attr('id', desc2Id);
                $desc2Input.attr('name', 'description2');
                $desc2Group.insertBefore($form.find('button'));
            }

            this._fillEventsSelect($form.find('select[name="event_type"]'));
            
            this.$form = $form;
        },
        
        _fillEventsSelect : function($select){
//          var $select = this.$formTemplate.find('select').empty();
            $select.empty();
            //fill event select combo box
            $.each(this.itemType.event_types, function( index, eventType ) {
                var $option = $('<option />');
                $option.attr('value', eventType.id);
                $option.attr('data-content', '<svg class=\'icon\'><use xlink:href=\'#'+ eventType.icon  +'\'></use></svg><span>'+ eventType.name +'</span>');
                $select.append($option);
            });
            
            if (this.itemType.event_types.length == 1){
                $select.val(this.itemType.event_types[0].id); //select first 
            }
        },

        _fillForm : function(){
            var obj = this.rowToEventObj(this.$editedRow);

            formValidator.deserializeForm(this.$form, {
                event_type: this.$editedRow.find('input[name="type-id"]').val(), 
                description1: obj.description1, 
                description2: obj.description2, 
            });
        },
        
        _serializeForm : function(){
            var ser = formValidator.serializeForm(this.$form);

            
            ser.description1 = ser.description1.trim();
            if (ser.description2) //might be not there
                ser.description2 = ser.description2.trim();
            
            return ser;
        }


    };

    window.modalEventsEditCtrl = modalEventsEditCtrl;
}($);