+function ($) {
    "use strict";

    var pageItemTypeEdit = {

        init: function () {
            this.$page = $('.page-item-type-edit');

            this.serviceMethod = 'saveItemType';
            this.formData = YourCalendar.item_type;


            this.$fields = this.$page.find('.fields');
            this.$fieldTemplate = this.$fields.find('.field').remove().removeClass('d-none');

            var self = this;
            this.$page.find('.dropdown[data-add-field] a.dropdown-item').click(function (event) {
                event.preventDefault();

                var type = $(this).attr('href');

                self.$fields.append(self.prepareFieldTemplate({type: type}, type));
                self.updateFieldsButtons();
            });

            this.$page.find('.icon-selector').iconSelector({
                multi: true,
                showSectionHeaders: true,
            });

            this.$page.find('button[data-pos-help]').click(function () {
                modalCtrl.fieldsPositions();
            });
            
            this.$page.find('button[data-desc-help]').click(function () {
                modalCtrl.descFields();
            });
            

            pageFormBase.init.apply(this, arguments); //call super
            pageBase.init.apply(this, arguments); //call super
        },
        
        additionalFormCheck : function(){

            
            var anyChecked = false;
            
            this.$page.find('input[name="event_types[]"]').each(function(){
               
                var checked = $(this).prop('checked');
                
                anyChecked = anyChecked || checked;
               
            });
            
            
            if (!anyChecked){
                formValidator.highlightForm(this.$form);
            }
            
            this.$page.find('.alert[data-no-events-selected]').toggleClass('d-none', anyChecked);
            
//            formValidator.highlightField(this.$page.find('input[name="event_types[][id]"]'));
            
            return anyChecked;
            
        },



        //this is called twice on start and after saving
        fillForm: function () {
            pageFormBase.fillForm.apply(this, arguments); //call super

            var self = this;
            
            self.$fields.empty();//remove all
            this.formData.field_types.forEach(function (fieldType){
//                self.prepareFieldTemplate(fieldType, fieldType.type);   
                self.$fields.append(self.prepareFieldTemplate(fieldType, fieldType.type));
                self.updateFieldsButtons();
            });
            
            this.formData.event_types.forEach(function (eventType){
                self.$page.find('input[name="event_types[]"][value="'+ eventType.id +'"]').prop('checked', true);
//                self.updateFieldsButtons();
            });
            
            
            //updates 
            this.$page.find('.icon-selector').iconSelector('update');
            
            formValidator.cleanFormDirty(this.$form);
        },

        serializeForm: function () {
            var serialized = pageFormBase.serializeForm.apply(this, arguments); //call super

            var key = 0,
                $cbx = this.$page.find('input[name^="event_types[]"]'),
                event_types = serialized.event_types;
        
            serialized.event_types = [];
            
            event_types.forEach(function(checked){
                
                if (checked === 1){
                    serialized.event_types.push({id: parseInt($cbx.eq(key).val())} ); //store event type id
                }
                
                key++;
            });
            
            if (!serialized.field_types) //if no fields , pass empty array needed
                serialized.field_types = []; 
            
            serialized.field_types.forEach(function(field_type){
                
                if (field_type.type != 'image'){
                    field_type.size = ''; //make empy coz not used
                }

            });            
            
            return serialized;
        },

        //additional things to check when validating form
        prepareFieldTemplate: function (formData, type) {
            var $field = this.$fieldTemplate.clone();

            //remove what is not used
            $field.find('[data-type][data-type!="' + type + '"]').remove();
            
            //hide field of allow filtering, but do not remove it, needed by validation
            if (type !== 'text'){
                $field.find('input[name="allow_filtering"]').closest('.form-group').addClass('d-none');
            }
            
            if (type !== 'image'){
                $field.find('select[name="size"]').closest('.form-group').addClass('d-none');
            }

            formValidator.deserializeForm($field, formData);

            // alert();
            var self = this;

            //add ids to inputs and for arguments to labels
            $field.find('input,select').each(function () {
                var $input = $(this);

                var randomId = 'a' + Math.floor(Math.random() * 9999);

                $input.attr('id', randomId);
                $input.siblings('label').attr('for', randomId);

                //preare names of input fields
                $input.attr('name', 'field_types[][' + $input.attr('name') + ']');
            });

            $field.find('button[data-delete]').click(function () {
                $field.remove();
                self.updateFieldsButtons();
            });


            $field.find('button[data-move-up]').click(function () {
                $field.prev().insertAfter($field);
                self.updateFieldsButtons();
                self.flashItem($field);
            });


            $field.find('button[data-move-down]').click(function () {
                $field.next().insertBefore($field);
                self.updateFieldsButtons();

                self.flashItem($field);
            });


            return $field;
        },

        updateFieldsButtons: function () {
            var $items = this.$fields.find('.field');

            var lastIndex = $items.length - 1;

            $items.each(function () {
                var $variant = $(this);
                var index = $variant.index();

                $variant.find('button[data-move-up]')[index == 0 ? 'attr' : 'removeAttr']('disabled', 1);
                $variant.find('button[data-move-down]')[index == lastIndex ? 'attr' : 'removeAttr']('disabled', 1);

            });
        },

        flashItem: function ($element) {
            $element.animate({opacity: 0}, 150).animate({opacity: 1}, 150);
        }

    };

    window.pageItemTypeEdit = $.extend(true, {}, pageBase, pageFormBase, pageItemTypeEdit); //must be deep copy;

}($);