/**
 * This file is part of Your Calendar app. It requires license to use.
 * Copyright (c) 2020 flexphperia.net All rights reserved.
 * 
 * @preserve
 */

+function ($) {
    'use strict';

    var IconSelector = function (element, options) {
        this.$element = null;
        this.$iconList = null;
        this.$selectedList = null;
        this.$input = null;

        this.options = null;
        this.selectedItemTemplate = '<a href="" ><span class="text-body" >&times;</span></a>';

        this.init(element, options);
    };

    IconSelector.prototype.init = function (element, options) {
        this.$element = $(element);
        this.$iconList = this.$element.find('.icon-list');
        this.$selectedList = this.$element.find('.selected-list');
        this.$input = this.$element.find('input[type="hidden"]');

        if (!this.$element.is('.icon-selector'))
            alert('Wrong element');

        this.options = this.getOptions(options);

        var self = this;

        this.$element.toggleClass('show-headers', this.options.showSectionHeaders);
        this.$element.toggleClass('multi', this.options.multi);

        this.$element.on('click.IconSelector', '.icon-list a', function (event, noInputUpdate) {
            event.preventDefault();
            // console.log('silent', silent);
            self.listClickHandler(this, noInputUpdate);
        });

        if (this.options.multi) {
            this.$element.on('click.IconSelector', '.selected-list a', function (event) {
                event.preventDefault();
                self.selectedListClickHandler(this);
            });
        }

        this.update();
    };


    IconSelector.prototype.getOptions = function (options) {
        var defaults = {
            multi: false,
            showSectionHeaders: true
        };

        //faster than jquery EXTEND
        for (var key in options) {
            defaults[key] = options[key];
        }

        return defaults;
    };

    //event handler
    IconSelector.prototype.listClickHandler = function (a, noInputUpdate) {
        var $a = $(a);

        // $a.addClass('selected');

        if (this.options.multi) {

            if ($a.hasClass('selected'))
                return;

            $a.addClass('selected');
            // console.log(1);
            var $newSelectedItem = $(this.selectedItemTemplate);
            var $newIcon = $a.find('.icon').clone();

            $newSelectedItem.prepend($newIcon);
            $newSelectedItem.attr('href', $a.attr('href'));

            this.$selectedList.append($newSelectedItem);
        } else {
            this.$iconList.find('.selected').removeClass('selected');
            $a.addClass('selected');
        }


        if (!noInputUpdate) {
            this._toInput();
            this._updateEmptyLabel();

        }
    };

    IconSelector.prototype.selectedListClickHandler = function (a) {
        var $a = $(a);

        var href = $a.attr('href');

        $a.remove();

        this.$iconList.find('a[href="' + href + '"]').removeClass('selected');

        this._toInput();
        this._updateEmptyLabel();
    };

    IconSelector.prototype._toInput = function () {
        var selectedNames = [];
        // if (this.options.multi){
        this.$iconList.find('a.selected').each(function () {
            selectedNames.push($(this).attr('href'));
        });
        this.$input.val(selectedNames.join(','));
    };

    IconSelector.prototype._fromInput = function () {
        var value = this.getValue(),
            self = this;
       
        if (!value) {
            return;
        }

        var names = value.split(',');

        names.forEach(function (name) {
            self.$iconList.find('a[href="' + name + '"]').trigger('click', [true]);
        });
    };


    IconSelector.prototype._scrollToSelected = function () {
        var $firstSelected = this.$iconList.find('.selected').eq(0);

        if (!$firstSelected.length)
            return;

        var pos = $firstSelected.position().top + this.$iconList.scrollTop();
//        console.log(this.$iconList.scrollTop());

        this.$iconList.scrollTop(pos - $firstSelected.outerHeight());
    };


    IconSelector.prototype._updateEmptyLabel = function () {
        // console.log(this.getValue().length);
        if (!this.options.multi)
            return;
        this.$selectedList.find('.empty').toggleClass('d-none', this.getValue().length ? true : false);
    };

    IconSelector.prototype.getValue = function () {
//         console.log(this.$input.get(0));
        return this.$input.val();
    };

    IconSelector.prototype.setValue = function (value) {
        this.$input.val(value);
        this._fromInput();
        this._scrollToSelected();
    };
    
    
    IconSelector.prototype.update = function () {
        this._fromInput();
        this._updateEmptyLabel();
        this._scrollToSelected();
        
//        console.log('update');
    };


    IconSelector.prototype.destroy = function () {
        //TODO
    };


    $.fn.iconSelector = function (option, param) {
        var $this = $(this).first(); //this plugin only gets first elment
        var data = $this.data('flexphperia.IconSelector');
        var options = typeof option == 'object' && option;

        if (!data && /destroy/.test(option))
            return; //destroy call return
        if (!data)
            $this.data('flexphperia.IconSelector', (data = new IconSelector(this, options))); // constructor call

        if (typeof option == 'string') {
            return data[option](param);
        } //if method call

        return this;
    };

    $.fn.iconSelector.Constructor = IconSelector;

}(jQuery);