+function ($) {
    'use strict';

    var Uploader = function (element, options) {
        this.$element = null;
		
        this.$button = null;
        this.$wrongExt = null;
        this.$error = null;
        this.$progress = null;
        this.$preview = null;
        this.$inputValue = null;
        this.$inputPreview = null;

        this.uploader = null;
		
        this.options = null;

        this.init(element, options);
    };

    Uploader.prototype.init = function (element, options) {
        this.$element = $(element);
        this.options = this.getOptions(options);

        var self = this;

        this.$button = this.$element.find('button[data-upload]'),
        this.$clear = this.$element.find('button[data-clear]'),
        this.$wrongExt = this.$element.find('[data-wrong-ext]'),
        this.$error = this.$element.find('[data-error]'),
        this.$progress = this.$element.find('.progress'),
        this.$preview = this.$element.find('.preview'),
        this.$previewImg = this.$element.find('.preview img'),
        this.$inputValue = this.$element.find('input[type="hidden"]'),
//		this.$inputPreview = this.$element.find('input[type="hidden"][name*="preview"]');

        this.uploader = new ss.SimpleUpload({
                button: this.$button, 
                url: this.options.url, 
                name: 'file', 
                responseType: 'json',
                allowedExtensions: this.options.allowedExtensions,
                accept: this.options.accept,
                hoverClass: 'hover',
                onChange :function( filename, extension, uploadBtn, fileSize, file ){
                     //set to zero used to reset progress bar when reused again after upload
                    self.$progress.find('.progress-bar').width(0 + '%').text(0 + '%');
                    self.$wrongExt.add(self.$error).add(self.$progress).addClass('d-none');
                    self.$inputValue.val('');
//               
                    self.updatePreview();
                },
                onExtError : function ( filename, extension ){
                    self.$wrongExt.removeClass('d-none').find('[data-ext]').text(extension);
                },
                onError : function( filename, errorType, status, statusText, response, uploadBtn, fileSize ){
                    self.$progress.addClass('d-none');
                    self.$error.removeClass('d-none').find('[data-status]').text(errorType + ', ' + statusText);
                },
                onProgress : function( pct ){
                    // console.log(pct);
                    self.$progress.find('.progress-bar').width(pct + '%').text(pct + '%');
                },
                onSubmit: function(filename, extension) {
                    self.$progress.removeClass('d-none');
                },         
                onComplete: function(filename, response) {
                    self.$progress.addClass('d-none');
                    self.$inputValue.val(response.data);
                    self.updatePreview('temp');
                }
        });  
        
        this.$clear.click(function(){
//            console.log(1);
            self.$inputValue.val('');
            self.updatePreview();
        });


        //get preview from input
        this.updatePreview();
    };


    Uploader.prototype.getOptions = function (options) {
        var defaults = {
            action : '',
            allowedExtensions: ['jpg','png'],
            accept: '.jpg,.png',
            previewWidth : 100,
        };

        //faster than jquery EXTEND
        for (var key in options) {
            defaults[key] = options[key];
        }
        
        defaults.url =  YourCalendar.siteUrl + 'upload/' + defaults.action;
        
//   console.log(defaults);
//        defaults.accept = defaults.allowedExtensions.jo
        

        return defaults;
    };

   	
    Uploader.prototype.updatePreview = function (dir) {
        dir = dir ? dir : 'uploads'; //default is uploads

        var src = this.$inputValue.val();
        this.$previewImg.toggleClass('d-none', src ? false : true);
        
        if (src){
            this.$previewImg.attr('src', YourCalendar.imgGeneratorUrl + '?dir=' + dir + '&width='+ this.options.previewWidth + '&filename=' + src);
        }
        
        this.$preview.find('.no-image').toggleClass('d-none', src ? true : false);
    };    
	
    Uploader.prototype.destroy = function () {
        this.uploader.destroy();
    };


    $.fn.uploader = function (option, param) {
        var $this = $(this).first(); //this plugin only gets first elment
        var data = $this.data('flexphperia.Uploader');
        var options = typeof option == 'object' && option;

        if (!data && /destroy/.test(option))
            return; //destroy call return
        if (!data)
            $this.data('flexphperia.Uploader', (data = new Uploader(this, options))); // constructor call

        if (typeof option == 'string') {
            return data[option](param);
        } //if method call

        return this;
    };

    $.fn.uploader.Constructor = Uploader;

}(jQuery);