+function ($) {
"use strict";

var selectionCtrl = {

    $element: null,
    $selectionPanel: null,

    selection: null,
    
    lastSelectionStop : 0,
    
    

    init: function ($element) {

        var self = this;

        this.$element = $element;
        this.$selectionPanel = $('#selectionPanel');

        this.$selectionPanel.find('.close').click(function () {
            self.clearSelection();
        });

        this.$selectionPanel.find('button[data-delete]').click(function () {
            modalCtrl.deleteEventsModal(function() {
                
                serviceCtrl.deleteEventsAll(self.getSelectionObjects(), function(result){
                    mainCtrl.showNotify(result[0]);
                    self.selection.getSelection().forEach(function (selected) {
                        $(selected).replaceWith(result[1]);
                        // obj.inst.removeFromSelection(selected);
                    });
                    
                    self.clearSelection();
                });
                
            });
        });


        this.$selectionPanel.find('button[data-add]').click(function () {

            var s = self.getSelectionObjects();
            
            var editOptions = {
                days : [],
                itemId: s[0].item_id //pasess first item id, to modal all selected items have same type
            };
            

            $.each(s, function( index, value ) {
                editOptions.days.push(value);
            });
 
             modalEventsEditCtrl.open(
                    'add',
                    function(events){
                        
                        var params = {
                            days : editOptions.days,
                            events: events
                        };
                        
                        serviceCtrl.addEvents(params, function(result){
                            mainCtrl.showNotify(result[0]);
                            
                            //fill cells
                            $.each(result[1], function( index, value ) {
//                                console.log(self.$element.find('.cali[data-year="'+value.year+'"][data-month="'+value.month+'"]').length);
//                                console.log(self.$element.find('.cali[data-year="'+value.year+'"][data-month="'+value.month+'"]').length);
                                self.$element
                                    .find('.cali[data-year="'+value.year+'"][data-month="'+value.month+'"]')
                                    .find('.item[data-item-id="'+value.item_id +'"] .day:eq('+ (value.day - 1) +') .events-list')
                                    .replaceWith(value.events);
                            });
                            self.clearSelection();
                        });
                        
                    },
                    null, //callback cancel
                    editOptions
            );
        });

        this.selection = new SelectionArea({
            // Class for the selection-area-element
            class: 'selection-area',

            // Enable single-click selection (Also disables range-selection via shift + ctrl)
//            singleClick: true,
            
            // Query selectors from elements which can be selected
            selectables: ['.item .events-list'],

            // Query selectors for elements from where a selection can be start
            startareas: [mainCtrl.touch ? '.day .events-list' : this.$element.get(0)],

            // Query selectors for elements which will be used as boundaries for the selection
            boundaries: [this.$element.get(0)],

            // Query selector or dom node to set up container for selection-area-element
            selectionAreaContainer: 'body',
            
            // Configuration in case a selectable gets just clicked.
            singleTap: {

                // Enable single-click selection (Also disables range-selection via shift + ctrl).
                allow: true,

                // 'native' (element was mouse-event target) or 'touch' (element visually touched).
                intersect: 'native'
            }

        })
        .on('start', function (obj) {
               //fix above header
            $('.selection-area').parent().css('z-index', 2);

//            console.log('start', obj.store.changed);
            // Remove class if the user isn't pressing the control key or command key
            if (!obj.event.ctrlKey && !obj.event.metaKey) {
                self.clearSelection();
            }

            self.$selectionPanel.removeClass('d-none');

//            console.log(this);
//            self.updateSelectionPanel(obj.store.stored);
        })
        .on('move', function (obj) {
            
//            console.log('move', obj.store.changed);
            // Add a custom class to the elements that where selected.
            obj.store.changed.added.forEach(function (added) {
                added.classList.add('selected');
            });

            // Remove the class from elements that where removed
            // since the last selection
            obj.store.changed.removed.forEach(function (removed) {
                removed.classList.remove('selected');
            });

            self.updateSelectionPanel(obj.store.stored.concat(obj.store.selected));
        })
        .on('stop', function (obj) {
//            console.log('stop', obj.store.changed)
            self.selection.keepSelection();
//            obj.inst.keepSelection();
            self.updateSelectionPanel(obj.store.stored);
            
            //cancel selection when clicked on one cell without CTRL key
            if ($('.selection-area').length == 0 && !obj.event.ctrlKey && !obj.event.metaKey ) {
                self.clearSelection();
                self.selection.cancel();
            }
            else{
            
                self.lastSelectionStop = new Date().getTime();
            }
        });
    },

    clearSelection: function () {
        // Unselect all elements
        this.selection.getSelection().forEach(function (selected) {
            selected.classList.remove('selected');
            // obj.inst.removeFromSelection(selected);
        });

        // Clear previous selection
        this.selection.clearSelection();

        this.$selectionPanel.addClass('d-none');

    },

    updateSelectionPanel: function (selectedArr) {

//    console.log(this.selection.getSelection());

        var previousType = null,
            multiTypes = false;


        selectedArr.forEach(function (element) {
            var $el = $(element);
            var type = $el.closest('.item').data('type-id');

//            console.log(previousType, type);
            //multi types selected
            if (previousType !== null && type !== previousType) {
                multiTypes = true;
            }

            previousType = type;
        });

        // console.log(multiTypes);
        this.$selectionPanel.find('[data-day-count]').text(selectedArr.length);

        this.$selectionPanel.find('[data-warning]').toggleClass('d-none', !multiTypes);
        this.$selectionPanel.find('[data-add]').prop('disabled', multiTypes || selectedArr.length == 0);
        this.$selectionPanel.find('[data-delete]').prop('disabled',selectedArr.length == 0);
    },

    disable: function () {
        this.selection.disable();
    },

    enable: function () {
        this.selection.enable();
    },
    
    getSelectionObjects : function(){
        
        var a = [];
        
//        console.log(this.selection.getSelection());
        
        this.selection.getSelection().forEach(function (element) {
            var $day = $(element).closest('.day');

            a.push({
                year: $day.closest('.cali').data('year'), 
                month: $day.closest('.cali').data('month'),
                day: $day.index() + 1,
                item_id : $day.closest('.item').data('item-id')
            });

            //only once
//            editOptions.itemId = !editOptions.itemId ? $day.closest('.item').data('item-id') : editOptions.itemId;
        });
        
        return a;
    },
    
    justStoppedSelection : function(){
//        console.log(new Date().getMilliseconds(),this.lastSelectionStop);
        return new Date().getTime() - this.lastSelectionStop < 50;
    }
};



window.selectionCtrl = selectionCtrl;

}($);