/*
 * Service controller, responsible for parsing server responses.
 * Every json response has two object keys: code  and data
 * 
 * Code - response status:
* 1 - ok
* 2 - error - return message to display in notify
* 3 - not logged in / session expired  
 */
var serviceCtrl = {
    
    //path to gate
    path : YourCalendar.siteUrl,
    
    
    init : function(){

    },
    
    //empty array, without string will be removed when jquery ajax call
    _fix_arrays : function (obj){
        $.each(obj, function(key, val){
            
            if ($.isArray(val) && val.length === 0)
            {
                obj[key] = '[|empty-array|]';
            }
            else if(val instanceof Object)
            {
                obj[key] = serviceCtrl._fix_arrays(val);
            }
        });
        
        return obj;
        
    },
    

    /* calls server method in specified controller in php script */
    callAjax : function (action, callbacks, data, withoutBlocker, errorCallback, type) {

        this._fix_arrays(data);

//data = [1,2,3];
//console.log(data);
        var self = this,
            opts = {
                url : this.path  + action,
                dataType : 'json',
                type : type ? type : 'POST',
                data: data,
                success: callbacks,
                error : function(jqXHR, textStatus, errorThrown) {
                    if (textStatus == 'abort')
                        return;
                    mainCtrl.showNotify(YourCalendar.lang['errorAjaxCallError'] + textStatus, 'danger');
                    uiBlockerCtrl.unblock();

                    errorCallback && errorCallback();
                }
            };

        opts.success = function(data, textStatus, jqXHR) {
            serviceCtrl.ajaxSuccess(data, callbacks, errorCallback, jqXHR);
        };
        
        var call = function(){
             $.ajax(opts);
        };
        
//        console.log(opts.url);

        if (!withoutBlocker){
            uiBlockerCtrl.block();
            setTimeout(function(){//after blocker shown
                call();
            }, 155);
            return;
        }
        
        
        call();
       
    },
    
    /* parses response, checks message code and calls appropiate callback or displays error
     * for codes see comments above
     */
    ajaxSuccess : function(data, successCallback, errorCallback, jqXHR){
        
        switch(data['code'])
        {
            case 1:
            {
                //call all callbacks
                if ($.isArray(successCallback)) {
                    $.each(successCallback, function(index, value) {
                        value(data.data, data.code, jqXHR);
                    });
                } else if (successCallback) {
                    successCallback(data.data, data.code, jqXHR);
                }

                uiBlockerCtrl.unblock();
                break;
            }   
            case 2: {
                mainCtrl.showNotify(data.data, 'danger'); //error (validation, resource not found, etc.)
                uiBlockerCtrl.unblock();
                errorCallback && errorCallback();
//                modalCtrl.loadingModal(false); //hide loading modal
                break;
            }  
            case 3: { //not logged in
                mainCtrl.showNotify(data.data, 'danger', true);
               //do not unblock
                errorCallback && errorCallback();
                break;
            }
            default: {
                alert('error wrong return code');
            }
        }
        
        if (data && data['profiler']){
            $('#ciProfiler').replaceWith (data['profiler']);
            
            //reopen profiler
            ci_profiler_bar && ci_profiler_bar.set_load_state();
        }
    },
    
    filter : function(data, viewSuccessCallback){	
        this.callAjax( 'filter', viewSuccessCallback, data );
    },
    
    
    getEventsForDay : function(data, viewSuccessCallback){	
        this.callAjax( 'events/get-for-day', viewSuccessCallback, data );
    },
    
    getItemType : function(id, viewSuccessCallback){		
        this.callAjax( 'events/get-item-type', viewSuccessCallback, {id: id} );
    },
    
    addEvents : function(data, viewSuccessCallback){
        //this data might be vary large so we take care to not exceed max_input_vars_limit
        data.days = JSON.stringify(data.days);
        
        this.callAjax( 'events/add', viewSuccessCallback, data );
    },
    
    saveEventsForDay : function(data, viewSuccessCallback){	
        this.callAjax( 'events/save', viewSuccessCallback, data );
    },
    
    deleteEventsAll : function(data, viewSuccessCallback){
        //this data might be vary large so we take care to not exceed max_input_vars_limit
        this.callAjax( 'events/delete-all', viewSuccessCallback, {cells: JSON.stringify(data)} );
    },

    
    saveItem : function(formData, viewSuccessCallback){		
        this.callAjax( 'items/save', viewSuccessCallback, formData );
    },
    
    deleteItem : function(id, viewSuccessCallback){		
        this.callAjax( 'items/delete', viewSuccessCallback, {id: id} );
    },
    
    updateItemTypesOrder : function(orders, viewSuccessCallback){		
        this.callAjax( 'item-types/update-order', viewSuccessCallback, orders );
    },
    
        
    
    saveItemType : function(formData, viewSuccessCallback){		
        this.callAjax( 'item-types/save', viewSuccessCallback, formData );
    },
    
    deleteItemType : function(id, viewSuccessCallback){		
        this.callAjax( 'item-types/delete', viewSuccessCallback, {id: id} );
    },
    
    saveEventType : function(formData, viewSuccessCallback){	
        this.callAjax( 'event-types/save', viewSuccessCallback, formData );
    },
    
    deleteEventType : function(id, viewSuccessCallback){		
        this.callAjax( 'event-types/delete', viewSuccessCallback, {id: id} );
    },
    
    saveUser : function(formData, viewSuccessCallback){		
        this.callAjax( 'users/save', viewSuccessCallback, formData );
    },
    
    deleteUser : function(id, viewSuccessCallback){		
        this.callAjax( 'users/delete', viewSuccessCallback, {id: id} );
    },
    
    saveUserPerms : function(formData, viewSuccessCallback){		
        this.callAjax( 'permissions/user-save', viewSuccessCallback, formData );
    },

    saveGroup : function(formData, viewSuccessCallback){		
        this.callAjax( 'permissions/group-save', viewSuccessCallback, formData );
    },
    
    deleteGroup : function(id, viewSuccessCallback){		
        this.callAjax( 'permissions/group-delete', viewSuccessCallback, {id: id} );
    },
    
    saveSettings : function(formData, viewSuccessCallback){		
        this.callAjax( 'settings/save', viewSuccessCallback, formData );
    },
	
    savePassword : function(formData, viewSuccessCallback){		
        this.callAjax( 'users/save-password', viewSuccessCallback, formData );
    }
    
};